SystemOrganization addCategory: #'System-CommandLine'!

Object subclass: #CommandLineArguments
	instanceVariableNames: 'arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

CommandLineArguments class
	instanceVariableNames: 'singleton'!

!CommandLineArguments commentStamp: '<historical>' prior: 0!
The CommandLineArguments represents the arguments passed to the image.
In the following case,

	$PHARO_VM myImage.image --foo bar
	
`CommandLineArguments default` contains {'--foo'. 'bar'}.!

CommandLineArguments class
	instanceVariableNames: 'singleton'!

!CommandLineArguments class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/2/2012 11:41'!
default
	^ singleton ifNil: [singleton := self new initialize]! !

!CommandLineArguments class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/1/2012 20:00'!
withArguments: aCollection
	^ self basicNew 
		initializeWithArguments: aCollection;
		yourself! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 12:33'!
allFilesWithExtension: anExtension
	^ self arguments select: [ :arg|
		arg endsWith: anExtension ]! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 19:57'!
argumentAt: index
	^ arguments at: index ! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:42'!
arguments
	^ arguments! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/13/2012 19:47'!
commandLineArguments
	"self commandLineArguments"
	
	| documentPath args |
	
	args := OrderedCollection withAll: Smalltalk arguments.
	documentPath := Smalltalk vm documentPath.
	documentPath isEmptyOrNil
		ifFalse: [ args addFirst: documentPath ].
 	^ args.! !

!CommandLineArguments methodsFor: 'copying' stamp: 'CamilloBruni 5/1/2012 20:07'!
copySubcommand
	"return a new copy of this CommandLine without the first arguments"
	^ self class withArguments: arguments allButFirst! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/1/2012 20:50'!
hasArguments
	^ arguments size > 0! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/2/2012 12:55'!
hasFileWithExtension: aFileExtension
	"return true if the first argument has the given file extension"
	^ self arguments anySatisfy: [ :arg| 
		arg endsWith: aFileExtension].! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 15:13'!
hasOption: aString
	| option |
	option := (aString size = 1
					ifTrue: [ '-']
					ifFalse: [ '--']), aString.
	(self arguments includes: option) ifTrue: [ ^ true ].
	option := option,'='.
	^ self arguments anySatisfy: [ :arg| arg beginsWith: option ].! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/2/2012 14:07'!
includesSubCommand: aName
	self withFirstArgument: [ :arg|
		arg = aName ifTrue: [ ^ true ]].
	^ false! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/13/2012 19:47'!
initialize
	
	| documentPath  |
	
	arguments := OrderedCollection withAll: Smalltalk arguments.
	documentPath := Smalltalk vm documentPath.
	documentPath isEmptyOrNil
		ifFalse: [ arguments addFirst: documentPath ].! !

!CommandLineArguments methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/1/2012 20:01'!
initializeWithArguments: aCollection
	super initialize.
	arguments := aCollection.! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 15:19'!
longOptionAt: aString
	| option optionStart |
	optionStart := '--', aString, '='.
	option := self arguments 
		detect: [ :arg| arg beginsWith: optionStart ] 
		ifNone: [ ^ nil ].
	^ (option splitOn: '=') second! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 15:24'!
optionAt: aString
	^ (aString size = 1)
		ifTrue: [ self shortOptionAt: aString ]
		ifFalse: [ self longOptionAt: aString ]! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 15:24'!
shortOptionAt: aString
	^ arguments at: (arguments indexOf: '-', aString) + 1! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 4/28/2012 19:54'!
withFirstArgument: aBlock
	self arguments ifEmpty: [ ^ self ].
	^ aBlock value: self arguments first! !

Object subclass: #CommandLineHandler
	instanceVariableNames: 'commandLine stdout stderr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!CommandLineHandler commentStamp: '<historical>' prior: 0!
A CommandLineHandler is activated by the CommandLine. 

The responsible handler with the highest priority is selected and its instance-side method #activate is invoked.

By default the handlers are selected by their class name. In the following shell invocation the FooHandler is chosen:

	$PATH_TO_VM myImage.image FooHandler

A handler may provide a short name with the class-side #commandName method. If the FooHandler defined #commandName returning 'foo' it would be activated with the following shell invocation:
	
	$PATH_TO_VM myImage.image foo
	
For more sophisticated handler selection the CommandLineHandler should implement the #isResponsibleFor: class-side method. An instance of the current command line options is passed to this method which should then return a boolean.

Between all the responsible handlers the one with the highes #priority is chosen. To change the priority overwrite the class-side accessor.
!

CommandLineHandler subclass: #BasicCodeLoader
	instanceVariableNames: 'sourceFiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!BasicCodeLoader commentStamp: '<historical>' prior: 0!
CodeLoader provides a way to load smalltalk source file from local.

Example: 

CodeLoader new 
	loadSourceFiles: (Array with: 
		(#file:, 
		FileDirectory default fullName, 
		'/useless.st')); 
	installSourceFiles !

!BasicCodeLoader class methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 11:35'!
commandName
	^ 'st'! !

!BasicCodeLoader class methodsFor: 'accessing' stamp: 'CamilloBruni 5/3/2012 14:14'!
description
	^ 'handles .st source files'! !

!BasicCodeLoader class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:31'!
isResponsibleFor: aCommandLine
	"This handler is reponsible only for .st files"
	(aCommandLine hasFileWithExtension: '.st')
		ifTrue: [ ^ true ].
	^ super isResponsibleFor: aCommandLine! !

!BasicCodeLoader methodsFor: 'activation' stamp: 'CamilloBruni 10/13/2012 15:50'!
activate
	self activateHelp.
	self loadSourceFiles: (self commandLine allFilesWithExtension: 'st').
	self installSourceFiles.
	(self commandLine hasOption: 'quit')
		ifTrue: [ self exitSuccess ].! !

!BasicCodeLoader methodsFor: 'private' stamp: 'CamilloBruni 4/28/2012 21:48'!
handleError: error	
	
	"for syntax errors we can used the default action"	"otherwise resignal it"
	(error isKindOf: SyntaxErrorNotification)
		ifTrue: [ error defaultAction ]
		ifFalse: [ error pass ]! !

!BasicCodeLoader methodsFor: 'private' stamp: 'EstebanLorenzano 4/4/2012 10:38'!
handleError: error reference: aReference
	"Print a header before failing on an errro / syntax notification from the the script loaded by the given request"
	|stderr|
	
	"spit out a warning if in headless mode, otherwise a debugger will popup"
	Smalltalk isHeadless 
		ifTrue: [ self inform: 'Errors in script loaded from ', aReference fullName ].
																														
	(error isKindOf: SyntaxErrorNotification)
		"for syntax errors we can used the default action"
		ifTrue: [ error defaultAction ]
		"otherwise resignal it"
		ifFalse: [ error pass ]! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'CamilloBruni 6/14/2012 23:15'!
handleErrorsDuring: aBlock reference: aReference 
	
	aBlock
		on: Error, ParserNotification
		do: [ :e| 	self handleError: e reference: aReference ].
	! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'CamilloBruni 6/14/2012 23:29'!
installSourceFile: aReference
	"Install the the source file given by aFileReference"
	| codeImporter |

	" parse the code given in the source file"
	aReference readStreamDo: [ :stream |
		self 
			handleErrorsDuring: [ 
				codeImporter := CodeImporter fileStream: stream.
				codeImporter parseDeclarations ]
			reference: aReference].
	
	"evaluate the declarations in a second step to allow for properly closing the source file stream"
	self 
		handleErrorsDuring: [ codeImporter evaluate ]
		reference: aReference.! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'EstebanLorenzano 4/4/2012 10:31'!
installSourceFiles
	"Install the previously loaded source files"

	sourceFiles ifNil: [ ^self ].
	[ sourceFiles do: [ :reference | self installSourceFile: reference ] ]
	ensure: [ sourceFiles := nil ].
	! !

!BasicCodeLoader methodsFor: 'loading' stamp: 'CamilloBruni 5/1/2012 20:51'!
loadSourceFiles: anArray
	"Load all the source files in the given array."

	sourceFiles := anArray collect: [ :each | 
						(FileSystem disk resolve: each) asFileReference ].
! !

BasicCodeLoader subclass: #EvaluateCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!EvaluateCommandLineHandler commentStamp: '<historical>' prior: 0!
Usage: eval [--help] <smalltalk expression>
	--help    list this help message
	<smallltalk expression>  a valid Smalltalk expression which is evaluated and 
	                         the result is printed on stdout

Documentation:
A CommandLineHandler that reads a string from the command line, outputs the evaluated result and quits the image. 

This handler either evaluates the arguments passed to the image:
	$PHARO_VM my.image eval  1 + 2
	
or it can read directly from stdin:

	echo "1+2" | $PHARO_VM my.image eval !

!EvaluateCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 22:48'!
commandName
	^ 'eval'! !

!EvaluateCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/3/2012 14:14'!
description
	^ 'directly evaluates passed in one line scripts'! !

!EvaluateCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 7/4/2012 00:20'!
isResponsibleFor: commandLineArguments
	"directly handle top-level -e and --evaluate options"
	commandLineArguments withFirstArgument: [ :arg| 
		(#('-e' '--evaluate') includes: arg)
			ifTrue: [ ^ true ]].
	
	^ commandLineArguments includesSubCommand: self commandName! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 10/13/2012 15:49'!
activate
	self activateHelp.
	self arguments ifEmpty: [ ^ self evaluateStdIn ].
	self evaluateArguments.
	self quit.! !

!EvaluateCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 6/23/2012 22:33'!
evaluate: aStream
		
	[ self stdout print: (Compiler evaluate: aStream); cr ]
		on: Error, ParserNotification
		do: [ :e| 	self handleError: e  ].! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 6/23/2012 22:46'!
evaluateArguments
	| args |
	args :=  self arguments joinUsing: Character space.
	args ifEmpty: [ ^ self ].
	self evaluate: args! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 6/23/2012 22:45'!
evaluateStdIn
	| stdin |
	stdin := FileStream stdin ifNil: [ ^ self ].
	stdin atEnd  ifTrue: [ ^ self ].
	
	self flag: #todo. "the whole compilation machinery does not work with stdin directly, hence we load everything a buffer first"
	self evaluate: (String streamContents: [ :s|
		[ stdin atEnd ] whileFalse: [ 
			stdin next ifNotNilDo: [ :char|
			s nextPut: char ]]])! !

!CommandLineHandler class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/2/2012 11:25'!
activateWith: aCommandLine 
	
	^ self new 
		commandLine: (self prepareSubcommand: aCommandLine);
		activate! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2012 17:38'!
allHandlers
	^ self allSubclasses reject: [ :handler| handler isAbstract ]! !

!CommandLineHandler class methodsFor: 'instance creation' stamp: 'CamilloBruni 10/13/2012 13:18'!
commandLine: aCommandLine 
	
	^ self new 
		commandLine: aCommandLine;
		yourself! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:56'!
commandName
	" Overwrite this method to provide a short name for this command handler.
	You may pass the commandName as first argument to the image to select this handler"
	^ self name! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:53'!
description
	"This method should return a short one-line description of the command"
	^ nil! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2012 17:39'!
isAbstract
	^ self = CommandLineHandler! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 14:06'!
isResponsibleFor: aCommandLineArguments
	^ aCommandLineArguments includesSubCommand: self commandName! !

!CommandLineHandler class methodsFor: 'private' stamp: 'CamilloBruni 5/2/2012 11:28'!
prepareSubcommand: commandLineArguments

	"strip the subcommand name from the arguments"
	commandLineArguments withFirstArgument: [ :arg|
		arg = self commandName 
			ifTrue: [ ^ commandLineArguments copySubcommand ]].
	
	"not a subcommand hence we keep the same args"
	^ commandLineArguments! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 01:51'!
priority
	^ 0! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:30'!
selectHandlersFor: aCommandLine

	^ self allHandlers select: [ :handlerClass|
		handlerClass isResponsibleFor: aCommandLine ]! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:33'!
<< aString
	^ self stdout 
		nextPutAll: aString;
		yourself! !

!CommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 4/28/2012 19:46'!
activate
	 self subclassResponsibility! !

!CommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 10/13/2012 15:52'!
activateHelp
	"Default help implementation, running #help if the only argument is --help "
	((self hasOption: 'help') and: [ self arguments size = 1 ])
		ifTrue: [ ^ self help ].! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:43'!
allHandlers 
	^ CommandLineHandler allHandlers ! !

!CommandLineHandler methodsFor: 'accessing arguments' stamp: 'CamilloBruni 10/13/2012 13:13'!
argumentAt: anInteger
	^ self commandLine argumentAt: anInteger! !

!CommandLineHandler methodsFor: 'accessing arguments' stamp: 'CamilloBruni 10/13/2012 13:13'!
arguments
	^ self commandLine arguments! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:28'!
commandLine
	^ commandLine! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:28'!
commandLine: aCommandLine 
	commandLine := aCommandLine! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:42'!
commandName
	^ self class commandName! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:52'!
description
	^ self class description! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 10/13/2012 15:44'!
exitFailure 
	^ self exitFailure: 'Commandline failed'! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 10/13/2012 15:44'!
exitFailure: aMessage 
	Smalltalk isInteractive 
		ifTrue: [ Error signal: aMessage ]
		ifFalse: [ Smalltalk exitFailure ].! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 10/13/2012 16:06'!
exitSuccess
	Smalltalk isInteractive 
		ifTrue: [ self inform: self class name, ' finished' ]
		ifFalse: [ Smalltalk exitSuccess ]! !

!CommandLineHandler methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 15:39'!
hasArguments
	^ self commandLine hasArguments! !

!CommandLineHandler methodsFor: 'accessing arguments' stamp: 'CamilloBruni 10/13/2012 13:13'!
hasOption: aString
	^ self commandLine hasOption: aString! !

!CommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 15:45'!
help
	"This is a crude default help implementation."
	self printHelp.
	Smalltalk isInteractive ifFalse: [ self exitSuccess ]! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 10/13/2012 13:45'!
initialize
	super initialize.
	self 
		initializeStdout;
		initializeStderr.
	! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 10/13/2012 13:32'!
initializeStderr
	"install the line end conversion and initialize the converter"
	FileStream stderr wantsLineEndConversion: true; converter.
	stderr := FileStream stderr .! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 4/28/2012 19:47'!
initializeStdout
	"install the line end conversion and initialize the converter"
	FileStream stdout wantsLineEndConversion: true; converter.
	stdout := FileStream stdout.! !

!CommandLineHandler methodsFor: 'accessing arguments' stamp: 'CamilloBruni 10/13/2012 14:07'!
optionAt: aString
	^ self commandLine optionAt: aString! !

!CommandLineHandler methodsFor: 'printing' stamp: 'CamilloBruni 10/13/2012 15:42'!
printHelp
	self stderr
		nextPutAll: self class comment.! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 7/25/2012 10:53'!
quit
	Smalltalk exitSuccess.! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:31'!
stderr
	^ stderr! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:47'!
stdout
	^ stdout! !

CommandLineHandler subclass: #ConfigurationCommandLineHandler
	instanceVariableNames: 'version repositoryURL configurationName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!ConfigurationCommandLineHandler commentStamp: '<historical>' prior: 0!
Usage: config [--help] <repository url> [<configuration>] [--install[=<version>]]
	--help              show this help message
	<repository url>    A Monticello repository name 
	<configuration>     A valid Metacello Configuration name
	<version>           A valid version for the given configuration
	
Examples:
	# display this help message
	$PharoVM My.image config
	
	# list all configurations of a repository
	$PharoVM My.image config $MC_REPOS_URL
	
	# list all the available versions of a confgurtation
	$PharoVM My.image config $MC_REPOS_URL ConfigurationOfFoo
	
	# install the stable version
	$PharoVM My.image config $MC_REPOS_URL ConfigurationOfFoo --install
	
	#install a specific version
	$PharoVM My.image config $MC_REPOS_URL ConfigurationOfFoo --install='1.5'!

!ConfigurationCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 12:51'!
commandName
	^ 'config'! !

!ConfigurationCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 12:51'!
description
	^ 'Install and inspect Metacello Configurations from the command line'! !

!ConfigurationCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 10/13/2012 15:50'!
activate
	self activateHelp.
	
	(self hasOption: 'install')
		ifFalse: [ ^ self list ].
		
	self installConfiguration.
		! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:55'!
configuration
	self gofer
		package: self configurationName;
		load.
	^ Smalltalk globals at: self configurationName asSymbol! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:54'!
configurationName
	^ configurationName ifNil: [ configurationName := self argumentAt: 2 ]! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 15:46'!
errorNoConfigurationsFound

	self exitFailure: 'No Configurations found in ', repositoryURL asString! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:53'!
gofer
	^ Gofer new 
		url: self repositoryURL;
		yourself! !

!ConfigurationCommandLineHandler methodsFor: 'testing' stamp: 'CamilloBruni 10/13/2012 13:20'!
hasConfiguration
	self flag: 'TODO: should use proper cli parser Coral??'.
	^ (self arguments size > 1) and: [
		((self argumentAt: 2) beginsWith: '-') not ]! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 14:12'!
installConfiguration
	self installVersion: ((self optionAt: 'install') ifNil: [ #stable ])! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 15:36'!
installVersion: aVersionName

	self inform: 'Installing ', self configurationName, ' ', aVersionName.
	(self project version: aVersionName) load.
	
	Smalltalk snapshot: true andQuit: true.! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 13:07'!
list
	self hasConfiguration 
		ifTrue: [ self listConfigurationDetails ]
		ifFalse: [ self listConfigurations ]! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 14:02'!
listConfigurationDetails
	"List possible configurations of the given repository"
	
	self printMetacelloProjectDetails: self project.
	
	^ self exitSuccess! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 13:42'!
listConfigurations
	"List possible configurations of the given repository"
	
	| configurations |
	
	configurations := self loadConfigurationNames.
	configurations ifEmpty: [ ^ self errorNoConfigurationsFound ].
	self printConfigurations: configurations.
	
	^ self exitSuccess! !

!ConfigurationCommandLineHandler methodsFor: 'actions' stamp: 'CamilloBruni 10/13/2012 13:53'!
loadConfigurationNames
	| references configurations |
	
	references := self gofer allResolved.
		
	references := references 
			collect: #packageName 
			thenSelect: [ :name| name beginsWith: 'ConfigurationOf' ].
	
	^ references asSet asSortedCollection ! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:25'!
loadRepositoryURL
	^ self argumentAt: 1.! !

!ConfigurationCommandLineHandler methodsFor: 'printing' stamp: 'CamilloBruni 10/13/2012 13:49'!
printConfigurations: configurations

	self inform: (String streamContents:  [ :s|
		s
			nextPutAll: 'Found ';
			print: configurations size;
			nextPutAll: ' Configuration';
			nextPutAll: (configurations size = 1 ifTrue: ':' ifFalse: 's:') ]).
	
	configurations do: [ :configurationName | 
			self << configurationName. 
			self stdout lf ].
	! !

!ConfigurationCommandLineHandler methodsFor: 'printing' stamp: 'CamilloBruni 10/13/2012 13:59'!
printMetacelloProjectDetails: metacelloProject
	self inform: 'Available versions for ', self configurationName.
	
	metacelloProject symbolicVersionMap keysDo: [ :key|
		self stdout print: key; lf ].
	
	metacelloProject map keysDo: [ :key|
		self stdout nextPutAll: key; lf ]! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 14:02'!
project
	^ self configuration project! !

!ConfigurationCommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 10/13/2012 13:24'!
repositoryURL
	^ repositoryURL ifNil: [ repositoryURL := self loadRepositoryURL ]! !

CommandLineHandler subclass: #DefaultCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!DefaultCommandLineHandler commentStamp: '<historical>' prior: 0!
Usage:
	--help       print this help message
	--copyright  print the copyrights
	--version    print the version for the image and the vm
	--list       list a description of all active command line handlers
	
Documentation:
A DefaultCommandLineHandler handles default command line arguments and options.
The DefaultCommandLineHandler is activated before all other handlers. 
It first checks if another handler is available. If so it will activate the found handler.!

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/3/2012 14:13'!
description
	^ 'responsible for the default options and activating other commands'! !

!DefaultCommandLineHandler class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/2/2012 11:34'!
initialize 
	"hook into the startup list"
	Smalltalk addToStartUpList: self.! !

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 01:27'!
isResponsibleFor: aCommandLine
	^ true! !

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:34'!
priority 
	"low priority"
	^ Float infinity negated! !

!DefaultCommandLineHandler class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/1/2012 21:18'!
startUp: resuming

	"only handle when lauching a new image"
	resuming ifFalse: [ ^ self ].
	Smalltalk addDeferredStartupAction: [ self new activate ]! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 7/24/2012 16:38'!
activate
	self arguments ifEmpty: [ ^ self ].
	
	^ self handleArgument: self arguments first.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 21:54'!
copyright
	self stdout 
		nextPutAll: Smalltalk licenseString; cr.
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 10/13/2012 15:47'!
error
	self arguments size = 1
		ifTrue:  [
			(self arguments first beginsWith: '-')
				ifTrue:  [ self << 'Invalid option: ']
				ifFalse: [ self << 'Invalid sub command: ']]
		ifFalse: [ self << 'Invalid arguments: "'].
		
	(self << (' ' join: self arguments)) 
		nextPutAll: '"'; cr.
		
	self printHelp.
	
	self exitFailure.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 00:24'!
handleArgument: aString
	
	"give priority to subcommands"
	self handleSubcommand == self
		ifFalse: [ ^ self ].
		
	"check for default options"
	aString = '--version'
		ifTrue: [ ^ self version ].
		
	aString = '--help'
		ifTrue: [ ^ self help ].
		
	aString = '--list'
		ifTrue: [ ^ self list ].
		
	aString = '--copyright'
		ifTrue: [ ^ self copyright ].
	
	"none of the previous options matched hence we output an error message"
	self error.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 00:24'!
handleSubcommand
	"check if there is an active subcommand"
	
	| handlers |
	
	handlers := self selectHandlers.
	
	"ignore the default command line handler"
	handlers first = DefaultCommandLineHandler
		ifTrue: [ ^ self ].
		
	^ handlers first activateWith: commandLine.! !

!DefaultCommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/1/2012 21:24'!
initialize
	super initialize.
	commandLine := CommandLineArguments new! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/26/2012 13:09'!
list
	| handlers shortNames maxShortNameSize |
	self stdout 
		nextPutAll: 'Currently installed Command Line Handlers:'; cr.
		
	handlers := self allHandlers reject: [ :cls| cls = DefaultCommandLineHandler ].
	shortNames := handlers collect: #commandName.
	maxShortNameSize := (shortNames detectMax: [ :name|  name size ]) size.
	
	handlers do: [ :handler|
		self stdout 
			nextPutAll: '    ';
			nextPutAll: (handler commandName padRightTo: maxShortNameSize); 
			nextPutAll: '    ';
			nextPutAll: (handler description ifNil: [ '--']);cr].
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'private' stamp: 'CamilloBruni 5/1/2012 21:22'!
selectHandlers
	| handlers |
	handlers := CommandLineHandler selectHandlersFor: commandLine.
	handlers := handlers sort: [ :a :b| a priority >= b priority ].
	^ handlers! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 22:00'!
version
	self stdout 
		nextPutAll: 'Image: '; print: SystemVersion current; cr;
		nextPutAll: 'VM:    '; nextPutAll: Smalltalk vm version; cr.
	self quit.! !

!CodeImporter methodsFor: '*System-CommandLine' stamp: 'CamilloBruni 6/14/2012 23:26'!
evaluate
	" stripped down version of evaluateDeclarations"
	| value |
	
	self codeDeclarations do: [ :decl | value := decl import ].
	self flushChangesFile.
	
	^value! !
DefaultCommandLineHandler initialize!
